# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class SCIMMigrationResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization_id': 'str',
        'total_users_processed': 'int',
        'users_migrated': 'list[UserMigrationResult]',
        'errors': 'list[str]',
        'users_to_remove': 'list[UserToRemove]',
        'dry_run': 'bool'
    }

    attribute_map = {
        'organization_id': 'organization_id',
        'total_users_processed': 'total_users_processed',
        'users_migrated': 'users_migrated',
        'errors': 'errors',
        'users_to_remove': 'users_to_remove',
        'dry_run': 'dry_run'
    }

    def __init__(self, organization_id=None, total_users_processed=None, users_migrated=None, errors=None, users_to_remove=None, dry_run=True, local_vars_configuration=None):  # noqa: E501
        """SCIMMigrationResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._organization_id = None
        self._total_users_processed = None
        self._users_migrated = None
        self._errors = None
        self._users_to_remove = None
        self._dry_run = None
        self.discriminator = None

        self.organization_id = organization_id
        self.total_users_processed = total_users_processed
        self.users_migrated = users_migrated
        self.errors = errors
        if users_to_remove is not None:
            self.users_to_remove = users_to_remove
        if dry_run is not None:
            self.dry_run = dry_run

    @property
    def organization_id(self):
        """Gets the organization_id of this SCIMMigrationResult.  # noqa: E501


        :return: The organization_id of this SCIMMigrationResult.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this SCIMMigrationResult.


        :param organization_id: The organization_id of this SCIMMigrationResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def total_users_processed(self):
        """Gets the total_users_processed of this SCIMMigrationResult.  # noqa: E501


        :return: The total_users_processed of this SCIMMigrationResult.  # noqa: E501
        :rtype: int
        """
        return self._total_users_processed

    @total_users_processed.setter
    def total_users_processed(self, total_users_processed):
        """Sets the total_users_processed of this SCIMMigrationResult.


        :param total_users_processed: The total_users_processed of this SCIMMigrationResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_users_processed is None:  # noqa: E501
            raise ValueError("Invalid value for `total_users_processed`, must not be `None`")  # noqa: E501

        self._total_users_processed = total_users_processed

    @property
    def users_migrated(self):
        """Gets the users_migrated of this SCIMMigrationResult.  # noqa: E501


        :return: The users_migrated of this SCIMMigrationResult.  # noqa: E501
        :rtype: list[UserMigrationResult]
        """
        return self._users_migrated

    @users_migrated.setter
    def users_migrated(self, users_migrated):
        """Sets the users_migrated of this SCIMMigrationResult.


        :param users_migrated: The users_migrated of this SCIMMigrationResult.  # noqa: E501
        :type: list[UserMigrationResult]
        """
        if self.local_vars_configuration.client_side_validation and users_migrated is None:  # noqa: E501
            raise ValueError("Invalid value for `users_migrated`, must not be `None`")  # noqa: E501

        self._users_migrated = users_migrated

    @property
    def errors(self):
        """Gets the errors of this SCIMMigrationResult.  # noqa: E501


        :return: The errors of this SCIMMigrationResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._errors

    @errors.setter
    def errors(self, errors):
        """Sets the errors of this SCIMMigrationResult.


        :param errors: The errors of this SCIMMigrationResult.  # noqa: E501
        :type: list[str]
        """
        if self.local_vars_configuration.client_side_validation and errors is None:  # noqa: E501
            raise ValueError("Invalid value for `errors`, must not be `None`")  # noqa: E501

        self._errors = errors

    @property
    def users_to_remove(self):
        """Gets the users_to_remove of this SCIMMigrationResult.  # noqa: E501


        :return: The users_to_remove of this SCIMMigrationResult.  # noqa: E501
        :rtype: list[UserToRemove]
        """
        return self._users_to_remove

    @users_to_remove.setter
    def users_to_remove(self, users_to_remove):
        """Sets the users_to_remove of this SCIMMigrationResult.


        :param users_to_remove: The users_to_remove of this SCIMMigrationResult.  # noqa: E501
        :type: list[UserToRemove]
        """

        self._users_to_remove = users_to_remove

    @property
    def dry_run(self):
        """Gets the dry_run of this SCIMMigrationResult.  # noqa: E501


        :return: The dry_run of this SCIMMigrationResult.  # noqa: E501
        :rtype: bool
        """
        return self._dry_run

    @dry_run.setter
    def dry_run(self, dry_run):
        """Sets the dry_run of this SCIMMigrationResult.


        :param dry_run: The dry_run of this SCIMMigrationResult.  # noqa: E501
        :type: bool
        """

        self._dry_run = dry_run

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SCIMMigrationResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SCIMMigrationResult):
            return True

        return self.to_dict() != other.to_dict()
