# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TableRedactionResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'table_name': 'str',
        'total_rows': 'int',
        'redacted_rows': 'int',
        'status': 'str',
        'error_message': 'str'
    }

    attribute_map = {
        'table_name': 'table_name',
        'total_rows': 'total_rows',
        'redacted_rows': 'redacted_rows',
        'status': 'status',
        'error_message': 'error_message'
    }

    def __init__(self, table_name=None, total_rows=None, redacted_rows=None, status=None, error_message=None, local_vars_configuration=None):  # noqa: E501
        """TableRedactionResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._table_name = None
        self._total_rows = None
        self._redacted_rows = None
        self._status = None
        self._error_message = None
        self.discriminator = None

        self.table_name = table_name
        self.total_rows = total_rows
        self.redacted_rows = redacted_rows
        self.status = status
        if error_message is not None:
            self.error_message = error_message

    @property
    def table_name(self):
        """Gets the table_name of this TableRedactionResult.  # noqa: E501


        :return: The table_name of this TableRedactionResult.  # noqa: E501
        :rtype: str
        """
        return self._table_name

    @table_name.setter
    def table_name(self, table_name):
        """Sets the table_name of this TableRedactionResult.


        :param table_name: The table_name of this TableRedactionResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and table_name is None:  # noqa: E501
            raise ValueError("Invalid value for `table_name`, must not be `None`")  # noqa: E501

        self._table_name = table_name

    @property
    def total_rows(self):
        """Gets the total_rows of this TableRedactionResult.  # noqa: E501


        :return: The total_rows of this TableRedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._total_rows

    @total_rows.setter
    def total_rows(self, total_rows):
        """Sets the total_rows of this TableRedactionResult.


        :param total_rows: The total_rows of this TableRedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_rows is None:  # noqa: E501
            raise ValueError("Invalid value for `total_rows`, must not be `None`")  # noqa: E501

        self._total_rows = total_rows

    @property
    def redacted_rows(self):
        """Gets the redacted_rows of this TableRedactionResult.  # noqa: E501


        :return: The redacted_rows of this TableRedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._redacted_rows

    @redacted_rows.setter
    def redacted_rows(self, redacted_rows):
        """Sets the redacted_rows of this TableRedactionResult.


        :param redacted_rows: The redacted_rows of this TableRedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and redacted_rows is None:  # noqa: E501
            raise ValueError("Invalid value for `redacted_rows`, must not be `None`")  # noqa: E501

        self._redacted_rows = redacted_rows

    @property
    def status(self):
        """Gets the status of this TableRedactionResult.  # noqa: E501


        :return: The status of this TableRedactionResult.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this TableRedactionResult.


        :param status: The status of this TableRedactionResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def error_message(self):
        """Gets the error_message of this TableRedactionResult.  # noqa: E501


        :return: The error_message of this TableRedactionResult.  # noqa: E501
        :rtype: str
        """
        return self._error_message

    @error_message.setter
    def error_message(self, error_message):
        """Sets the error_message of this TableRedactionResult.


        :param error_message: The error_message of this TableRedactionResult.  # noqa: E501
        :type: str
        """

        self._error_message = error_message

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TableRedactionResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TableRedactionResult):
            return True

        return self.to_dict() != other.to_dict()
