# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class BackendServerApiProductModelsCatalogClientModelsTableMetadata(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'catalog_name': 'str',
        'schema_name': 'str',
        'table_type': 'str',
        'data_source_format': 'str',
        'description': 'str',
        'owner': 'str',
        'created_at': 'int',
        'updated_at': 'int',
        'full_name': 'str',
        'columns': 'list[ColumnInfo]',
        'storage_location': 'str',
        'properties': 'object',
        'version': 'str',
        'row_count': 'int',
        'size_bytes': 'int'
    }

    attribute_map = {
        'name': 'name',
        'catalog_name': 'catalog_name',
        'schema_name': 'schema_name',
        'table_type': 'table_type',
        'data_source_format': 'data_source_format',
        'description': 'description',
        'owner': 'owner',
        'created_at': 'created_at',
        'updated_at': 'updated_at',
        'full_name': 'full_name',
        'columns': 'columns',
        'storage_location': 'storage_location',
        'properties': 'properties',
        'version': 'version',
        'row_count': 'row_count',
        'size_bytes': 'size_bytes'
    }

    def __init__(self, name=None, catalog_name=None, schema_name=None, table_type=None, data_source_format=None, description=None, owner=None, created_at=0, updated_at=0, full_name=None, columns=None, storage_location=None, properties=None, version=None, row_count=None, size_bytes=None, local_vars_configuration=None):  # noqa: E501
        """BackendServerApiProductModelsCatalogClientModelsTableMetadata - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._catalog_name = None
        self._schema_name = None
        self._table_type = None
        self._data_source_format = None
        self._description = None
        self._owner = None
        self._created_at = None
        self._updated_at = None
        self._full_name = None
        self._columns = None
        self._storage_location = None
        self._properties = None
        self._version = None
        self._row_count = None
        self._size_bytes = None
        self.discriminator = None

        self.name = name
        self.catalog_name = catalog_name
        self.schema_name = schema_name
        self.table_type = table_type
        if data_source_format is not None:
            self.data_source_format = data_source_format
        if description is not None:
            self.description = description
        if owner is not None:
            self.owner = owner
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if full_name is not None:
            self.full_name = full_name
        if columns is not None:
            self.columns = columns
        if storage_location is not None:
            self.storage_location = storage_location
        if properties is not None:
            self.properties = properties
        if version is not None:
            self.version = version
        if row_count is not None:
            self.row_count = row_count
        if size_bytes is not None:
            self.size_bytes = size_bytes

    @property
    def name(self):
        """Gets the name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Name of the table  # noqa: E501

        :return: The name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Name of the table  # noqa: E501

        :param name: The name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def catalog_name(self):
        """Gets the catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Name of the parent catalog  # noqa: E501

        :return: The catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._catalog_name

    @catalog_name.setter
    def catalog_name(self, catalog_name):
        """Sets the catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Name of the parent catalog  # noqa: E501

        :param catalog_name: The catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and catalog_name is None:  # noqa: E501
            raise ValueError("Invalid value for `catalog_name`, must not be `None`")  # noqa: E501

        self._catalog_name = catalog_name

    @property
    def schema_name(self):
        """Gets the schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Name of the parent schema  # noqa: E501

        :return: The schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._schema_name

    @schema_name.setter
    def schema_name(self, schema_name):
        """Sets the schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Name of the parent schema  # noqa: E501

        :param schema_name: The schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and schema_name is None:  # noqa: E501
            raise ValueError("Invalid value for `schema_name`, must not be `None`")  # noqa: E501

        self._schema_name = schema_name

    @property
    def table_type(self):
        """Gets the table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Type of table (e.g., 'MANAGED', 'EXTERNAL', 'VIEW')  # noqa: E501

        :return: The table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._table_type

    @table_type.setter
    def table_type(self, table_type):
        """Sets the table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Type of table (e.g., 'MANAGED', 'EXTERNAL', 'VIEW')  # noqa: E501

        :param table_type: The table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and table_type is None:  # noqa: E501
            raise ValueError("Invalid value for `table_type`, must not be `None`")  # noqa: E501

        self._table_type = table_type

    @property
    def data_source_format(self):
        """Gets the data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Data format of the table (e.g., 'DELTA', 'PARQUET')  # noqa: E501

        :return: The data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._data_source_format

    @data_source_format.setter
    def data_source_format(self, data_source_format):
        """Sets the data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Data format of the table (e.g., 'DELTA', 'PARQUET')  # noqa: E501

        :param data_source_format: The data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._data_source_format = data_source_format

    @property
    def description(self):
        """Gets the description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Description or comment for the table  # noqa: E501

        :return: The description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Description or comment for the table  # noqa: E501

        :param description: The description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def owner(self):
        """Gets the owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Owner of the table  # noqa: E501

        :return: The owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._owner

    @owner.setter
    def owner(self, owner):
        """Sets the owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Owner of the table  # noqa: E501

        :param owner: The owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._owner = owner

    @property
    def created_at(self):
        """Gets the created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Creation timestamp in milliseconds since epoch  # noqa: E501

        :return: The created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Creation timestamp in milliseconds since epoch  # noqa: E501

        :param created_at: The created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Last update timestamp in milliseconds since epoch  # noqa: E501

        :return: The updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Last update timestamp in milliseconds since epoch  # noqa: E501

        :param updated_at: The updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._updated_at = updated_at

    @property
    def full_name(self):
        """Gets the full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Fully qualified name of the table  # noqa: E501

        :return: The full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._full_name

    @full_name.setter
    def full_name(self, full_name):
        """Sets the full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Fully qualified name of the table  # noqa: E501

        :param full_name: The full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._full_name = full_name

    @property
    def columns(self):
        """Gets the columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        List of columns in the table  # noqa: E501

        :return: The columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: list[ColumnInfo]
        """
        return self._columns

    @columns.setter
    def columns(self, columns):
        """Sets the columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        List of columns in the table  # noqa: E501

        :param columns: The columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: list[ColumnInfo]
        """

        self._columns = columns

    @property
    def storage_location(self):
        """Gets the storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Physical storage location of the table data  # noqa: E501

        :return: The storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._storage_location

    @storage_location.setter
    def storage_location(self, storage_location):
        """Sets the storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Physical storage location of the table data  # noqa: E501

        :param storage_location: The storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._storage_location = storage_location

    @property
    def properties(self):
        """Gets the properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Additional table properties  # noqa: E501

        :return: The properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: object
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Additional table properties  # noqa: E501

        :param properties: The properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: object
        """

        self._properties = properties

    @property
    def version(self):
        """Gets the version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Version identifier for versioned tables (e.g., Delta Lake)  # noqa: E501

        :return: The version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Version identifier for versioned tables (e.g., Delta Lake)  # noqa: E501

        :param version: The version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._version = version

    @property
    def row_count(self):
        """Gets the row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Number of rows in the table  # noqa: E501

        :return: The row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._row_count

    @row_count.setter
    def row_count(self, row_count):
        """Sets the row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Number of rows in the table  # noqa: E501

        :param row_count: The row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._row_count = row_count

    @property
    def size_bytes(self):
        """Gets the size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Size of the table data in bytes  # noqa: E501

        :return: The size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._size_bytes

    @size_bytes.setter
    def size_bytes(self, size_bytes):
        """Sets the size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Size of the table data in bytes  # noqa: E501

        :param size_bytes: The size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._size_bytes = size_bytes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BackendServerApiProductModelsCatalogClientModelsTableMetadata):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, BackendServerApiProductModelsCatalogClientModelsTableMetadata):
            return True

        return self.to_dict() != other.to_dict()
