# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class RedactionResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization_id': 'str',
        'started_at': 'datetime',
        'completed_at': 'datetime',
        'success': 'bool',
        'dry_run': 'bool',
        'total_tables_processed': 'int',
        'total_rows_examined': 'int',
        'total_rows_redacted': 'int',
        'table_results': 'list[TableRedactionResult]',
        'errors': 'list[str]'
    }

    attribute_map = {
        'organization_id': 'organization_id',
        'started_at': 'started_at',
        'completed_at': 'completed_at',
        'success': 'success',
        'dry_run': 'dry_run',
        'total_tables_processed': 'total_tables_processed',
        'total_rows_examined': 'total_rows_examined',
        'total_rows_redacted': 'total_rows_redacted',
        'table_results': 'table_results',
        'errors': 'errors'
    }

    def __init__(self, organization_id=None, started_at=None, completed_at=None, success=None, dry_run=None, total_tables_processed=None, total_rows_examined=None, total_rows_redacted=None, table_results=None, errors=None, local_vars_configuration=None):  # noqa: E501
        """RedactionResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._organization_id = None
        self._started_at = None
        self._completed_at = None
        self._success = None
        self._dry_run = None
        self._total_tables_processed = None
        self._total_rows_examined = None
        self._total_rows_redacted = None
        self._table_results = None
        self._errors = None
        self.discriminator = None

        self.organization_id = organization_id
        self.started_at = started_at
        self.completed_at = completed_at
        self.success = success
        self.dry_run = dry_run
        self.total_tables_processed = total_tables_processed
        self.total_rows_examined = total_rows_examined
        self.total_rows_redacted = total_rows_redacted
        self.table_results = table_results
        if errors is not None:
            self.errors = errors

    @property
    def organization_id(self):
        """Gets the organization_id of this RedactionResult.  # noqa: E501


        :return: The organization_id of this RedactionResult.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this RedactionResult.


        :param organization_id: The organization_id of this RedactionResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def started_at(self):
        """Gets the started_at of this RedactionResult.  # noqa: E501


        :return: The started_at of this RedactionResult.  # noqa: E501
        :rtype: datetime
        """
        return self._started_at

    @started_at.setter
    def started_at(self, started_at):
        """Sets the started_at of this RedactionResult.


        :param started_at: The started_at of this RedactionResult.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and started_at is None:  # noqa: E501
            raise ValueError("Invalid value for `started_at`, must not be `None`")  # noqa: E501

        self._started_at = started_at

    @property
    def completed_at(self):
        """Gets the completed_at of this RedactionResult.  # noqa: E501


        :return: The completed_at of this RedactionResult.  # noqa: E501
        :rtype: datetime
        """
        return self._completed_at

    @completed_at.setter
    def completed_at(self, completed_at):
        """Sets the completed_at of this RedactionResult.


        :param completed_at: The completed_at of this RedactionResult.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and completed_at is None:  # noqa: E501
            raise ValueError("Invalid value for `completed_at`, must not be `None`")  # noqa: E501

        self._completed_at = completed_at

    @property
    def success(self):
        """Gets the success of this RedactionResult.  # noqa: E501


        :return: The success of this RedactionResult.  # noqa: E501
        :rtype: bool
        """
        return self._success

    @success.setter
    def success(self, success):
        """Sets the success of this RedactionResult.


        :param success: The success of this RedactionResult.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and success is None:  # noqa: E501
            raise ValueError("Invalid value for `success`, must not be `None`")  # noqa: E501

        self._success = success

    @property
    def dry_run(self):
        """Gets the dry_run of this RedactionResult.  # noqa: E501


        :return: The dry_run of this RedactionResult.  # noqa: E501
        :rtype: bool
        """
        return self._dry_run

    @dry_run.setter
    def dry_run(self, dry_run):
        """Sets the dry_run of this RedactionResult.


        :param dry_run: The dry_run of this RedactionResult.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and dry_run is None:  # noqa: E501
            raise ValueError("Invalid value for `dry_run`, must not be `None`")  # noqa: E501

        self._dry_run = dry_run

    @property
    def total_tables_processed(self):
        """Gets the total_tables_processed of this RedactionResult.  # noqa: E501


        :return: The total_tables_processed of this RedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._total_tables_processed

    @total_tables_processed.setter
    def total_tables_processed(self, total_tables_processed):
        """Sets the total_tables_processed of this RedactionResult.


        :param total_tables_processed: The total_tables_processed of this RedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_tables_processed is None:  # noqa: E501
            raise ValueError("Invalid value for `total_tables_processed`, must not be `None`")  # noqa: E501

        self._total_tables_processed = total_tables_processed

    @property
    def total_rows_examined(self):
        """Gets the total_rows_examined of this RedactionResult.  # noqa: E501


        :return: The total_rows_examined of this RedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._total_rows_examined

    @total_rows_examined.setter
    def total_rows_examined(self, total_rows_examined):
        """Sets the total_rows_examined of this RedactionResult.


        :param total_rows_examined: The total_rows_examined of this RedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_rows_examined is None:  # noqa: E501
            raise ValueError("Invalid value for `total_rows_examined`, must not be `None`")  # noqa: E501

        self._total_rows_examined = total_rows_examined

    @property
    def total_rows_redacted(self):
        """Gets the total_rows_redacted of this RedactionResult.  # noqa: E501


        :return: The total_rows_redacted of this RedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._total_rows_redacted

    @total_rows_redacted.setter
    def total_rows_redacted(self, total_rows_redacted):
        """Sets the total_rows_redacted of this RedactionResult.


        :param total_rows_redacted: The total_rows_redacted of this RedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_rows_redacted is None:  # noqa: E501
            raise ValueError("Invalid value for `total_rows_redacted`, must not be `None`")  # noqa: E501

        self._total_rows_redacted = total_rows_redacted

    @property
    def table_results(self):
        """Gets the table_results of this RedactionResult.  # noqa: E501


        :return: The table_results of this RedactionResult.  # noqa: E501
        :rtype: list[TableRedactionResult]
        """
        return self._table_results

    @table_results.setter
    def table_results(self, table_results):
        """Sets the table_results of this RedactionResult.


        :param table_results: The table_results of this RedactionResult.  # noqa: E501
        :type: list[TableRedactionResult]
        """
        if self.local_vars_configuration.client_side_validation and table_results is None:  # noqa: E501
            raise ValueError("Invalid value for `table_results`, must not be `None`")  # noqa: E501

        self._table_results = table_results

    @property
    def errors(self):
        """Gets the errors of this RedactionResult.  # noqa: E501


        :return: The errors of this RedactionResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._errors

    @errors.setter
    def errors(self, errors):
        """Sets the errors of this RedactionResult.


        :param errors: The errors of this RedactionResult.  # noqa: E501
        :type: list[str]
        """

        self._errors = errors

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RedactionResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RedactionResult):
            return True

        return self.to_dict() != other.to_dict()
