# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class MachinePoolSearchResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'machine_pool_name': 'str',
        'machine_pool_id': 'str',
        'organization_id': 'str',
        'cloud_ids': 'list[str]',
        'cloud_resource_ids': 'dict(str, list[str])',
        'spec': 'object'
    }

    attribute_map = {
        'machine_pool_name': 'machine_pool_name',
        'machine_pool_id': 'machine_pool_id',
        'organization_id': 'organization_id',
        'cloud_ids': 'cloud_ids',
        'cloud_resource_ids': 'cloud_resource_ids',
        'spec': 'spec'
    }

    def __init__(self, machine_pool_name=None, machine_pool_id=None, organization_id=None, cloud_ids=None, cloud_resource_ids=None, spec=None, local_vars_configuration=None):  # noqa: E501
        """MachinePoolSearchResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._machine_pool_name = None
        self._machine_pool_id = None
        self._organization_id = None
        self._cloud_ids = None
        self._cloud_resource_ids = None
        self._spec = None
        self.discriminator = None

        self.machine_pool_name = machine_pool_name
        self.machine_pool_id = machine_pool_id
        self.organization_id = organization_id
        self.cloud_ids = cloud_ids
        if cloud_resource_ids is not None:
            self.cloud_resource_ids = cloud_resource_ids
        if spec is not None:
            self.spec = spec

    @property
    def machine_pool_name(self):
        """Gets the machine_pool_name of this MachinePoolSearchResult.  # noqa: E501

        The name of the machine pool.  # noqa: E501

        :return: The machine_pool_name of this MachinePoolSearchResult.  # noqa: E501
        :rtype: str
        """
        return self._machine_pool_name

    @machine_pool_name.setter
    def machine_pool_name(self, machine_pool_name):
        """Sets the machine_pool_name of this MachinePoolSearchResult.

        The name of the machine pool.  # noqa: E501

        :param machine_pool_name: The machine_pool_name of this MachinePoolSearchResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and machine_pool_name is None:  # noqa: E501
            raise ValueError("Invalid value for `machine_pool_name`, must not be `None`")  # noqa: E501

        self._machine_pool_name = machine_pool_name

    @property
    def machine_pool_id(self):
        """Gets the machine_pool_id of this MachinePoolSearchResult.  # noqa: E501

        The unique identifier of the machine pool.  # noqa: E501

        :return: The machine_pool_id of this MachinePoolSearchResult.  # noqa: E501
        :rtype: str
        """
        return self._machine_pool_id

    @machine_pool_id.setter
    def machine_pool_id(self, machine_pool_id):
        """Sets the machine_pool_id of this MachinePoolSearchResult.

        The unique identifier of the machine pool.  # noqa: E501

        :param machine_pool_id: The machine_pool_id of this MachinePoolSearchResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and machine_pool_id is None:  # noqa: E501
            raise ValueError("Invalid value for `machine_pool_id`, must not be `None`")  # noqa: E501

        self._machine_pool_id = machine_pool_id

    @property
    def organization_id(self):
        """Gets the organization_id of this MachinePoolSearchResult.  # noqa: E501

        The organization ID that owns the machine pool.  # noqa: E501

        :return: The organization_id of this MachinePoolSearchResult.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this MachinePoolSearchResult.

        The organization ID that owns the machine pool.  # noqa: E501

        :param organization_id: The organization_id of this MachinePoolSearchResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def cloud_ids(self):
        """Gets the cloud_ids of this MachinePoolSearchResult.  # noqa: E501

        The cloud IDs that are attached to the machine pool.  # noqa: E501

        :return: The cloud_ids of this MachinePoolSearchResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._cloud_ids

    @cloud_ids.setter
    def cloud_ids(self, cloud_ids):
        """Sets the cloud_ids of this MachinePoolSearchResult.

        The cloud IDs that are attached to the machine pool.  # noqa: E501

        :param cloud_ids: The cloud_ids of this MachinePoolSearchResult.  # noqa: E501
        :type: list[str]
        """
        if self.local_vars_configuration.client_side_validation and cloud_ids is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_ids`, must not be `None`")  # noqa: E501

        self._cloud_ids = cloud_ids

    @property
    def cloud_resource_ids(self):
        """Gets the cloud_resource_ids of this MachinePoolSearchResult.  # noqa: E501

        The cloud resource IDs for each cloud ID that is attached to the machine pool.  # noqa: E501

        :return: The cloud_resource_ids of this MachinePoolSearchResult.  # noqa: E501
        :rtype: dict(str, list[str])
        """
        return self._cloud_resource_ids

    @cloud_resource_ids.setter
    def cloud_resource_ids(self, cloud_resource_ids):
        """Sets the cloud_resource_ids of this MachinePoolSearchResult.

        The cloud resource IDs for each cloud ID that is attached to the machine pool.  # noqa: E501

        :param cloud_resource_ids: The cloud_resource_ids of this MachinePoolSearchResult.  # noqa: E501
        :type: dict(str, list[str])
        """

        self._cloud_resource_ids = cloud_resource_ids

    @property
    def spec(self):
        """Gets the spec of this MachinePoolSearchResult.  # noqa: E501

        A machine pool specification; schema and allowed fields are defined in Anyscale documentation.  # noqa: E501

        :return: The spec of this MachinePoolSearchResult.  # noqa: E501
        :rtype: object
        """
        return self._spec

    @spec.setter
    def spec(self, spec):
        """Sets the spec of this MachinePoolSearchResult.

        A machine pool specification; schema and allowed fields are defined in Anyscale documentation.  # noqa: E501

        :param spec: The spec of this MachinePoolSearchResult.  # noqa: E501
        :type: object
        """

        self._spec = spec

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MachinePoolSearchResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MachinePoolSearchResult):
            return True

        return self.to_dict() != other.to_dict()
