# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ClusterEvent(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'message': 'str',
        'created_at': 'datetime',
        'metadata': 'object',
        'level': 'EventLevel'
    }

    attribute_map = {
        'message': 'message',
        'created_at': 'created_at',
        'metadata': 'metadata',
        'level': 'level'
    }

    def __init__(self, message=None, created_at=None, metadata=None, level=None, local_vars_configuration=None):  # noqa: E501
        """ClusterEvent - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._message = None
        self._created_at = None
        self._metadata = None
        self._level = None
        self.discriminator = None

        self.message = message
        self.created_at = created_at
        if metadata is not None:
            self.metadata = metadata
        self.level = level

    @property
    def message(self):
        """Gets the message of this ClusterEvent.  # noqa: E501


        :return: The message of this ClusterEvent.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this ClusterEvent.


        :param message: The message of this ClusterEvent.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and message is None:  # noqa: E501
            raise ValueError("Invalid value for `message`, must not be `None`")  # noqa: E501

        self._message = message

    @property
    def created_at(self):
        """Gets the created_at of this ClusterEvent.  # noqa: E501


        :return: The created_at of this ClusterEvent.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this ClusterEvent.


        :param created_at: The created_at of this ClusterEvent.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def metadata(self):
        """Gets the metadata of this ClusterEvent.  # noqa: E501


        :return: The metadata of this ClusterEvent.  # noqa: E501
        :rtype: object
        """
        return self._metadata

    @metadata.setter
    def metadata(self, metadata):
        """Sets the metadata of this ClusterEvent.


        :param metadata: The metadata of this ClusterEvent.  # noqa: E501
        :type: object
        """

        self._metadata = metadata

    @property
    def level(self):
        """Gets the level of this ClusterEvent.  # noqa: E501

        The level of the event  # noqa: E501

        :return: The level of this ClusterEvent.  # noqa: E501
        :rtype: EventLevel
        """
        return self._level

    @level.setter
    def level(self, level):
        """Sets the level of this ClusterEvent.

        The level of the event  # noqa: E501

        :param level: The level of this ClusterEvent.  # noqa: E501
        :type: EventLevel
        """
        if self.local_vars_configuration.client_side_validation and level is None:  # noqa: E501
            raise ValueError("Invalid value for `level`, must not be `None`")  # noqa: E501

        self._level = level

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ClusterEvent):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ClusterEvent):
            return True

        return self.to_dict() != other.to_dict()
