# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class UsageByCloud(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'anyscale_credits': 'float',
        'dollar_value': 'float',
        'date': 'date',
        'cloud_id': 'str',
        'cloud_name': 'str'
    }

    attribute_map = {
        'anyscale_credits': 'anyscale_credits',
        'dollar_value': 'dollar_value',
        'date': 'date',
        'cloud_id': 'cloud_id',
        'cloud_name': 'cloud_name'
    }

    def __init__(self, anyscale_credits=None, dollar_value=None, date=None, cloud_id=None, cloud_name=None, local_vars_configuration=None):  # noqa: E501
        """UsageByCloud - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._anyscale_credits = None
        self._dollar_value = None
        self._date = None
        self._cloud_id = None
        self._cloud_name = None
        self.discriminator = None

        self.anyscale_credits = anyscale_credits
        if dollar_value is not None:
            self.dollar_value = dollar_value
        if date is not None:
            self.date = date
        self.cloud_id = cloud_id
        self.cloud_name = cloud_name

    @property
    def anyscale_credits(self):
        """Gets the anyscale_credits of this UsageByCloud.  # noqa: E501


        :return: The anyscale_credits of this UsageByCloud.  # noqa: E501
        :rtype: float
        """
        return self._anyscale_credits

    @anyscale_credits.setter
    def anyscale_credits(self, anyscale_credits):
        """Sets the anyscale_credits of this UsageByCloud.


        :param anyscale_credits: The anyscale_credits of this UsageByCloud.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and anyscale_credits is None:  # noqa: E501
            raise ValueError("Invalid value for `anyscale_credits`, must not be `None`")  # noqa: E501

        self._anyscale_credits = anyscale_credits

    @property
    def dollar_value(self):
        """Gets the dollar_value of this UsageByCloud.  # noqa: E501


        :return: The dollar_value of this UsageByCloud.  # noqa: E501
        :rtype: float
        """
        return self._dollar_value

    @dollar_value.setter
    def dollar_value(self, dollar_value):
        """Sets the dollar_value of this UsageByCloud.


        :param dollar_value: The dollar_value of this UsageByCloud.  # noqa: E501
        :type: float
        """

        self._dollar_value = dollar_value

    @property
    def date(self):
        """Gets the date of this UsageByCloud.  # noqa: E501


        :return: The date of this UsageByCloud.  # noqa: E501
        :rtype: date
        """
        return self._date

    @date.setter
    def date(self, date):
        """Sets the date of this UsageByCloud.


        :param date: The date of this UsageByCloud.  # noqa: E501
        :type: date
        """

        self._date = date

    @property
    def cloud_id(self):
        """Gets the cloud_id of this UsageByCloud.  # noqa: E501


        :return: The cloud_id of this UsageByCloud.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this UsageByCloud.


        :param cloud_id: The cloud_id of this UsageByCloud.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def cloud_name(self):
        """Gets the cloud_name of this UsageByCloud.  # noqa: E501


        :return: The cloud_name of this UsageByCloud.  # noqa: E501
        :rtype: str
        """
        return self._cloud_name

    @cloud_name.setter
    def cloud_name(self, cloud_name):
        """Sets the cloud_name of this UsageByCloud.


        :param cloud_name: The cloud_name of this UsageByCloud.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_name is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_name`, must not be `None`")  # noqa: E501

        self._cloud_name = cloud_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UsageByCloud):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UsageByCloud):
            return True

        return self.to_dict() != other.to_dict()
