# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ColumnInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'type_text': 'str',
        'type_name': 'str',
        'description': 'str',
        'nullable': 'bool',
        'position': 'int'
    }

    attribute_map = {
        'name': 'name',
        'type_text': 'type_text',
        'type_name': 'type_name',
        'description': 'description',
        'nullable': 'nullable',
        'position': 'position'
    }

    def __init__(self, name=None, type_text=None, type_name=None, description=None, nullable=True, position=0, local_vars_configuration=None):  # noqa: E501
        """ColumnInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._type_text = None
        self._type_name = None
        self._description = None
        self._nullable = None
        self._position = None
        self.discriminator = None

        self.name = name
        self.type_text = type_text
        self.type_name = type_name
        if description is not None:
            self.description = description
        if nullable is not None:
            self.nullable = nullable
        if position is not None:
            self.position = position

    @property
    def name(self):
        """Gets the name of this ColumnInfo.  # noqa: E501

        Name of the column  # noqa: E501

        :return: The name of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ColumnInfo.

        Name of the column  # noqa: E501

        :param name: The name of this ColumnInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def type_text(self):
        """Gets the type_text of this ColumnInfo.  # noqa: E501

        Data type in text format (e.g., 'string', 'int', 'array<string>')  # noqa: E501

        :return: The type_text of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._type_text

    @type_text.setter
    def type_text(self, type_text):
        """Sets the type_text of this ColumnInfo.

        Data type in text format (e.g., 'string', 'int', 'array<string>')  # noqa: E501

        :param type_text: The type_text of this ColumnInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and type_text is None:  # noqa: E501
            raise ValueError("Invalid value for `type_text`, must not be `None`")  # noqa: E501

        self._type_text = type_text

    @property
    def type_name(self):
        """Gets the type_name of this ColumnInfo.  # noqa: E501

        Data type name in uppercase (e.g., 'STRING', 'INT')  # noqa: E501

        :return: The type_name of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._type_name

    @type_name.setter
    def type_name(self, type_name):
        """Sets the type_name of this ColumnInfo.

        Data type name in uppercase (e.g., 'STRING', 'INT')  # noqa: E501

        :param type_name: The type_name of this ColumnInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and type_name is None:  # noqa: E501
            raise ValueError("Invalid value for `type_name`, must not be `None`")  # noqa: E501

        self._type_name = type_name

    @property
    def description(self):
        """Gets the description of this ColumnInfo.  # noqa: E501

        Description or comment for the column  # noqa: E501

        :return: The description of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ColumnInfo.

        Description or comment for the column  # noqa: E501

        :param description: The description of this ColumnInfo.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def nullable(self):
        """Gets the nullable of this ColumnInfo.  # noqa: E501

        Whether the column allows NULL values  # noqa: E501

        :return: The nullable of this ColumnInfo.  # noqa: E501
        :rtype: bool
        """
        return self._nullable

    @nullable.setter
    def nullable(self, nullable):
        """Sets the nullable of this ColumnInfo.

        Whether the column allows NULL values  # noqa: E501

        :param nullable: The nullable of this ColumnInfo.  # noqa: E501
        :type: bool
        """

        self._nullable = nullable

    @property
    def position(self):
        """Gets the position of this ColumnInfo.  # noqa: E501

        Position of the column in the table schema  # noqa: E501

        :return: The position of this ColumnInfo.  # noqa: E501
        :rtype: int
        """
        return self._position

    @position.setter
    def position(self, position):
        """Sets the position of this ColumnInfo.

        Position of the column in the table schema  # noqa: E501

        :param position: The position of this ColumnInfo.  # noqa: E501
        :type: int
        """

        self._position = position

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ColumnInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ColumnInfo):
            return True

        return self.to_dict() != other.to_dict()
