# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OrganizationUserGroupsCollaborator(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'org_id': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'permission_level': 'OrganizationPermissionLevel',
        'members': 'list[MiniUser]',
        'member_count': 'int'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'org_id': 'org_id',
        'created_at': 'created_at',
        'updated_at': 'updated_at',
        'permission_level': 'permission_level',
        'members': 'members',
        'member_count': 'member_count'
    }

    def __init__(self, id=None, name=None, org_id=None, created_at=None, updated_at=None, permission_level=None, members=None, member_count=None, local_vars_configuration=None):  # noqa: E501
        """OrganizationUserGroupsCollaborator - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._org_id = None
        self._created_at = None
        self._updated_at = None
        self._permission_level = None
        self._members = None
        self._member_count = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.org_id = org_id
        self.created_at = created_at
        self.updated_at = updated_at
        if permission_level is not None:
            self.permission_level = permission_level
        if members is not None:
            self.members = members
        self.member_count = member_count

    @property
    def id(self):
        """Gets the id of this OrganizationUserGroupsCollaborator.  # noqa: E501

        The user group ID  # noqa: E501

        :return: The id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrganizationUserGroupsCollaborator.

        The user group ID  # noqa: E501

        :param id: The id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this OrganizationUserGroupsCollaborator.  # noqa: E501

        The user group name  # noqa: E501

        :return: The name of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrganizationUserGroupsCollaborator.

        The user group name  # noqa: E501

        :param name: The name of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def org_id(self):
        """Gets the org_id of this OrganizationUserGroupsCollaborator.  # noqa: E501


        :return: The org_id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: str
        """
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Sets the org_id of this OrganizationUserGroupsCollaborator.


        :param org_id: The org_id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and org_id is None:  # noqa: E501
            raise ValueError("Invalid value for `org_id`, must not be `None`")  # noqa: E501

        self._org_id = org_id

    @property
    def created_at(self):
        """Gets the created_at of this OrganizationUserGroupsCollaborator.  # noqa: E501


        :return: The created_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this OrganizationUserGroupsCollaborator.


        :param created_at: The created_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this OrganizationUserGroupsCollaborator.  # noqa: E501


        :return: The updated_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this OrganizationUserGroupsCollaborator.


        :param updated_at: The updated_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_at is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_at`, must not be `None`")  # noqa: E501

        self._updated_at = updated_at

    @property
    def permission_level(self):
        """Gets the permission_level of this OrganizationUserGroupsCollaborator.  # noqa: E501

        The organization-level role for this group  # noqa: E501

        :return: The permission_level of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: OrganizationPermissionLevel
        """
        return self._permission_level

    @permission_level.setter
    def permission_level(self, permission_level):
        """Sets the permission_level of this OrganizationUserGroupsCollaborator.

        The organization-level role for this group  # noqa: E501

        :param permission_level: The permission_level of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: OrganizationPermissionLevel
        """

        self._permission_level = permission_level

    @property
    def members(self):
        """Gets the members of this OrganizationUserGroupsCollaborator.  # noqa: E501

        List of group members  # noqa: E501

        :return: The members of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: list[MiniUser]
        """
        return self._members

    @members.setter
    def members(self, members):
        """Sets the members of this OrganizationUserGroupsCollaborator.

        List of group members  # noqa: E501

        :param members: The members of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: list[MiniUser]
        """

        self._members = members

    @property
    def member_count(self):
        """Gets the member_count of this OrganizationUserGroupsCollaborator.  # noqa: E501

        Total number of members  # noqa: E501

        :return: The member_count of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: int
        """
        return self._member_count

    @member_count.setter
    def member_count(self, member_count):
        """Sets the member_count of this OrganizationUserGroupsCollaborator.

        Total number of members  # noqa: E501

        :param member_count: The member_count of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and member_count is None:  # noqa: E501
            raise ValueError("Invalid value for `member_count`, must not be `None`")  # noqa: E501

        self._member_count = member_count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrganizationUserGroupsCollaborator):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrganizationUserGroupsCollaborator):
            return True

        return self.to_dict() != other.to_dict()
